@@ -241,7 +241,7 @@ applet_menu_deactivate (GtkWidget *w, AppletInfo *info)
 
 	panel_widget = PANEL_WIDGET (info->widget->parent);
 
-	panel_toplevel_unblock_auto_hide (panel_widget->toplevel);
+	panel_toplevel_pop_autohide_disabler (panel_widget->toplevel);
 }
 
 AppletUserMenu *
@@ -529,7 +529,7 @@ applet_show_menu (AppletInfo     *info,
 	if (info->menu == NULL)
 		return;
 
-	panel_toplevel_block_auto_hide (panel_widget->toplevel);
+	panel_toplevel_push_autohide_disabler (panel_widget->toplevel);
 
 	panel_applet_menu_set_recurse (GTK_MENU (info->menu),
 				       "menu_panel",
@@ -790,9 +790,9 @@ panel_applet_load_idle_handler (gpointer dummy)
 }
 
 void
-panel_applet_queue_applet_to_load (char            *id,
+panel_applet_queue_applet_to_load (const char      *id,
 				   PanelObjectType  type,
-				   char            *toplevel_id,
+				   const char      *toplevel_id,
 				   int              position,
 				   gboolean         right_stick,
 				   gboolean         locked)
@@ -801,15 +801,14 @@ panel_applet_queue_applet_to_load (char            *id,
 
 	if (!toplevel_id) {
 		g_warning ("No toplevel on which to load object '%s'\n", id);
-		g_free (id);
 		return;
 	}
 
 	applet = g_new0 (PanelAppletToLoad, 1);
 
-	applet->id          = id;
+	applet->id          = g_strdup (id);
 	applet->type        = type;
-	applet->toplevel_id = toplevel_id;
+	applet->toplevel_id = g_strdup (toplevel_id);
 	applet->position    = position;
 	applet->right_stick = right_stick != FALSE;
 	applet->locked      = locked != FALSE;
